package ru.myitschool.work.data

import ru.myitschool.work.data.dto.UserDto
import ru.myitschool.work.data.local.CredentialsLocalDataSource
import ru.myitschool.work.data.local.UserLocalDataSource
import ru.myitschool.work.domain.entities.UserEntity
import ru.myitschool.work.domain.login.LoginRepository
import ru.myitschool.work.data.network.UserNetworkDataSource
import ru.myitschool.work.domain.user.UserRepository

class UserRepositoryImpl(
    private val credentialsLocalDataSource: CredentialsLocalDataSource,
    private val userLocalDataSource: UserLocalDataSource,
    private val networkDataSource: UserNetworkDataSource
) : LoginRepository, UserRepository {

    override suspend fun login(login: String, password: String): Result<Unit> {

        return runCatching {
            networkDataSource.login(credentialsLocalDataSource.updateToken(login, password))
                .fold(
                    onSuccess = { dto ->
                        map(dto).fold(
                            onSuccess = {
                                userLocalDataSource.cacheData(it)
                            },
                            onFailure = { error(it) }
                        )
                    },
                    onFailure = { error(it) }
                )
        }
    }

    override suspend fun authorize(token: String): Result<Unit> {
        return networkDataSource.login(token).fold(
            onSuccess = { dto ->
                map(dto).fold(
                    onSuccess = { Result.success(userLocalDataSource.cacheData(it)) },
                    onFailure = { Result.failure(it) }
                )
            },
            onFailure = { Result.failure(it) }
        )
    }

    override suspend fun logout() {
        credentialsLocalDataSource.clear()
        userLocalDataSource.clear()
    }

    override suspend fun isUserExist(login: String): Result<Boolean> {
        return networkDataSource.isUserExist(login)
    }

    override suspend fun getCurrentUser(): UserEntity {
        return userLocalDataSource.getUser()!!
    }

    private suspend fun map(userDto: UserDto): Result<UserEntity> {
        return runCatching {
            UserEntity(
                name = userDto.name ?: error("Null user name"),
                lastVisit = userDto.lastVisit ?: "",
                photoUrl = userDto.photoUrl ?: error("Null user photoUrl"),
                position = userDto.position ?: error("Null user position"),
                isAdmin = userDto.roleId?.let { it ->
                    networkDataSource.isRoleHasAdminPermissions(
                        it,
                        credentialsLocalDataSource.getToken()
                    ).fold(onSuccess = { it }, onFailure = { error(it) })
                } ?: error("Null user roleId"),
                isCardBlocked = userDto.isCardBlocked ?: error("Null user isCardBlocked")
            )
        }
    }

    override suspend fun getUserByLogin(login: String): Result<UserEntity> {
        return networkDataSource.getUserByLogin(login, credentialsLocalDataSource.getToken()).fold(
            onSuccess = { map(it) }, onFailure = { Result.failure(it) }
        )
    }
}