package ru.myitschool.work.ui.login

import android.app.Application
import android.util.Log
import androidx.lifecycle.AndroidViewModel
import androidx.lifecycle.ViewModel
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.viewModelScope
import androidx.lifecycle.viewmodel.CreationExtras
import kotlinx.coroutines.channels.BufferOverflow
import kotlinx.coroutines.channels.Channel
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.flow.receiveAsFlow
import kotlinx.coroutines.launch
import ru.myitschool.work.data.UserRepositoryImpl
import ru.myitschool.work.data.local.CredentialsLocalDataSource
import ru.myitschool.work.data.local.UserLocalDataSource
import ru.myitschool.work.data.network.UserNetworkDataSource
import ru.myitschool.work.domain.login.AuthorizeUseCase
import ru.myitschool.work.domain.login.IsUserExistUseCase
import ru.myitschool.work.domain.login.LoginUseCase

class LoginViewModel(
    private val loginUseCase: LoginUseCase,
    private val isUserExistUseCase: IsUserExistUseCase,
    private val authorizeUseCase: AuthorizeUseCase,
    private val application: Application
) : AndroidViewModel(application) {

    private val _state = MutableStateFlow<State>(State.Waiting)
    private val _action = Channel<Action>(
        capacity = Channel.BUFFERED,
        onBufferOverflow = BufferOverflow.DROP_OLDEST
    )
    val state = _state.asStateFlow()
    val action = _action.receiveAsFlow()
    private var userCheckCompleted = false
    private var currentLogin: String? = null

    init {
        viewModelScope.launch {
            CredentialsLocalDataSource.getInstance().getTokenForAuth().fold(
                onSuccess = { token ->
                    authorizeUseCase(token).fold(
                        onSuccess = { _action.send(Action.OpenApp) },
                        onFailure = { _action.send(Action.GoToLogin) })
                },
                onFailure = { _action.send(Action.GoToLogin) }
            )
        }
    }

    fun changeLogin(login: String) {
        viewModelScope.launch {
            currentLogin = login
            userCheckCompleted = login.isNotBlank() && login.length >= 3 &&
                    !Character.isDigit(login[0]) &&
                    login.matches("^[a-zA-Z0-9]+$".toRegex())

            _state.emit(State.LoginCheckCompleted(userCheckCompleted))
        }
    }

    fun onProcessClick(login: String, password: String) {
        viewModelScope.launch {
            _state.emit(State.Loading)
            isUserExistUseCase.invoke(login).fold(
                onSuccess = { response ->
                    if (response) {
                        loginUseCase.invoke(login, password).fold(
                            onSuccess = { openApp() },
                            onFailure = { error ->
                                _state.emit(State.Error(error.message.toString()))
                            }
                        )
                    } else {
                        _state.emit(State.Error("No such user or incorrect"))
                    }
                },
                onFailure = { error -> _state.emit(State.Error(error.message.toString())) }
            )
        }
    }

    private fun openApp() {
        viewModelScope.launch {
            _action.send(Action.OpenApp)
        }
    }

    sealed interface State {
        data object Waiting : State
        data object Loading : State
        data class Error(val errorMessage: String) : State
        data class LoginCheckCompleted(val isCompleted: Boolean) : State
    }

    sealed interface Action {
        data object OpenApp : Action
        data object GoToLogin : Action
    }

    companion object {
        val Factory: ViewModelProvider.Factory = object : ViewModelProvider.Factory {
            @Suppress("UNCHECKED_CAST")
            override fun <T : ViewModel> create(modelClass: Class<T>, extras: CreationExtras): T {
                val application =
                    extras[ViewModelProvider.AndroidViewModelFactory.APPLICATION_KEY]!!

                val repository = UserRepositoryImpl(
                    userLocalDataSource = UserLocalDataSource,
                    credentialsLocalDataSource = CredentialsLocalDataSource.getInstance(),
                    networkDataSource = UserNetworkDataSource
                )

                return LoginViewModel(
                    loginUseCase = LoginUseCase(repository),
                    isUserExistUseCase = IsUserExistUseCase(repository),
                    authorizeUseCase = AuthorizeUseCase(repository),
                    application = application
                ) as T
            }
        }
    }
}