package ru.myitschool.work.ui.profile

import android.util.Log
import androidx.lifecycle.ViewModel
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.viewModelScope
import androidx.lifecycle.viewmodel.CreationExtras
import androidx.paging.Pager
import androidx.paging.PagingConfig
import androidx.paging.cachedIn
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.launch
import ru.myitschool.work.data.PassRepositoryImpl
import ru.myitschool.work.data.UserRepositoryImpl
import ru.myitschool.work.data.local.CredentialsLocalDataSource
import ru.myitschool.work.data.local.UserLocalDataSource
import ru.myitschool.work.data.network.PassNetworkDataSource
import ru.myitschool.work.data.network.UserNetworkDataSource
import ru.myitschool.work.domain.entities.PassEntity
import ru.myitschool.work.domain.entities.UserEntity
import ru.myitschool.work.domain.login.LogoutUseCase
import ru.myitschool.work.domain.passes.GetCurrentPassesUseCase
import ru.myitschool.work.domain.user.GetCurrentUserUseCase

class UserViewModel(
    private val getCurrentUserUseCase: GetCurrentUserUseCase,
    private val logoutUseCase: LogoutUseCase,
    private val getCurrentPassesUseCase: GetCurrentPassesUseCase
) : ViewModel() {

    private val _state = MutableStateFlow<State>(State.Loading)
    val state = _state.asStateFlow()

    val listState = Pager(
        config = PagingConfig(
            pageSize = 10,
            enablePlaceholders = false,
            maxSize = 50
        )
    ) {
        PassesPagingSource(getCurrentPassesUseCase::invoke)
    }.flow
        .cachedIn(viewModelScope)

    init {
        updateState()
    }

    private fun updateState() {
        viewModelScope.launch {
            _state.emit(State.Loading)

            _state.emit(
                State.Show(
                    getCurrentUserUseCase(),
                    getCurrentPassesUseCase(0, 30).getOrNull()!!
                )
            )

        }
    }

    fun onRefresh() {
        updateState()
    }

    fun onLogout() {
        viewModelScope.launch { logoutUseCase() }
    }

    sealed interface State {
        data object Loading : State
        data class Show(val userEntity: UserEntity, val passes: List<PassEntity>) : State
    }

    companion object {
        val Factory: ViewModelProvider.Factory = object : ViewModelProvider.Factory {
            @Suppress("UNCHECKED_CAST")
            override fun <T : ViewModel> create(modelClass: Class<T>, extras: CreationExtras): T {
                val repository = UserRepositoryImpl(
                    credentialsLocalDataSource = CredentialsLocalDataSource.getInstance(),
                    userLocalDataSource = UserLocalDataSource,
                    networkDataSource = UserNetworkDataSource
                )
                return UserViewModel(
                    getCurrentUserUseCase = GetCurrentUserUseCase(repository = repository),
                    logoutUseCase = LogoutUseCase(repository = repository),
                    getCurrentPassesUseCase = GetCurrentPassesUseCase(
                        repository = PassRepositoryImpl(
                            networkDataSource = PassNetworkDataSource,
                            credentialsLocalDataSource = CredentialsLocalDataSource.getInstance()
                        )
                    )
                ) as T
            }
        }
    }
}